% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rankcluster-package.R
\docType{package}
\name{Rankcluster-package}
\alias{Rankcluster-package}
\title{Model-Based Clustering for Multivariate Partial Ranking Data}
\description{
This package proposes a model-based clustering algorithm for ranking data.
Multivariate rankings as well as partial rankings are taken into account.
This algorithm is based on an extension of the Insertion Sorting Rank (ISR) model for ranking data, which is a meaningful
and effective model parametrized by a position parameter (the modal ranking, quoted by mu) and a dispersion parameter
(quoted by pi). The heterogeneity of the rank population is modeled by a mixture of ISR, whereas conditional independence
assumption is considered for multivariate rankings.
}
\details{
The main function is \link{rankclust}.
See vignettes for detailed examples: \code{RShowDoc("dataFormat", package = "Rankcluster")} and
\code{RShowDoc("Rankcluster", package = "Rankcluster")}
}
\examples{
# see vignettes
# RShowDoc("dataFormat", package = "Rankcluster")
# RShowDoc("Rankcluster", package = "Rankcluster")

# main function of the package for run the algorithm
data(big4)
result <- rankclust(big4$data, K = 2, m = big4$m, Ql = 200, Bl = 100, maxTry = 2)

if(result@convergence) {
  summary(result)

  partition <- result[2]@partition
  tik <- result[2]@tik
}

}
\references{
[1] C.Biernacki and J.Jacques (2013), A generative model for rank data based on sorting algorithm,
Computational Statistics and Data Analysis, 58, 162-176.

[2] J.Jacques and C.Biernacki (2012), Model-based clustering for multivariate partial ranking data,
Inria Research Report n 8113.
}
\keyword{package}
